{-----------------------------------------------------------------------
*  TIconButton v1.0                                                    *
*                                                                      *
*    Copyright  1996, 1997 InforTech, Inc.  All Rights Reserved.      *
*    Developed By: Steven Costa Martins                                *
*    E-MAIL: steven.martins@workmail.com                               *
						 infortech@usa.net                                         *
*    Home Page:                                                        *
*    http://www.geocities.com/SiliconValley/Park/7619/                 *
*                                                                      *
*    Last Modification: 23 Dec 1996, 10:21 AM                          *
------------------------------------------------------------------------
*  You are not allowed to redistribute this                            *
*  code or any part of it                                              *
------------------------------------------------------------------------}

unit Iconbutn;

interface

{$A+} { Word align data }
{$B-} { Complete Boolean Evaluation Directive }
{$D-} { Debug Information Directive }
{$L-} { Local Symbol Information Directive }
{$R-} { Range checking off }
{$S-} { Stack checking off }
{$T-} { We don't need (nor want) this type checking! }

{$IFDEF WIN32}
uses Messages, Windows, SysUtils, Classes,
	Controls, Forms, Menus, Graphics, Dialogs, DsgnIntf;
{$ELSE}
uses WinTypes, WinProcs, Messages, SysUtils, Classes,
	Controls, Forms, Menus, Graphics, Dialogs, DsgnIntf;
{$ENDIF}

type
  (*
   *  Property AboutBox
   *)
  TAboutProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  (*
   *  IconButton
   *)
  TButtonState = (bsUp, bsDisabled, bsDown, bsExclusive);
  TIconButton = class(TCustomControl)
    private
      FAbout: TAboutProperty;
    	FIcon: TIcon;
	    FDown: Boolean;
			FDragging: Boolean;
			FState: TButtonState;
    	procedure AutoInitialize;
    	procedure AutoDestroy;
    	procedure SetIcon(Value: TIcon);
    protected
    	procedure Click; override;
    	procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
			procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    	procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    	procedure Paint; override;
    	procedure DoDrawText(cv: TCanvas; x,y,w,h: Integer);
    	procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    	procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    	procedure RaisedBorderOutlineFill(cv: TCanvas; r: TRect; c: TColor);
    	procedure SunkenBorderOutlineFill(cv: TCanvas; r: TRect; c: TColor);
    	procedure SunkenBorder(cv: TCanvas; r: TRect);
    public
  		constructor Create(AOwner: TComponent); override;
    	destructor Destroy; override;
    	procedure DrawUp;
    	procedure DrawDown;
    	procedure CMTextChanged(var msg: TMessage);message CM_TEXTCHANGED;
    	procedure CMFocusChanged(var msg: TMessage);message CM_FOCUSCHANGED;
    	procedure CMEnabledChanged(var msg: TMessage);message CM_ENABLEDCHANGED;
    	procedure CMDialogChar(var Message: TCMDialogChar);message CM_DIALOGCHAR;
    published
			property About: TAboutProperty read FAbout write FAbout;
    	property Icon: TIcon read FIcon write SetIcon;
    	property Font;
    	property TabStop default true;
    	property TabOrder;
    	property Caption;
    	property Visible;
    	property Enabled;
    	property ParentFont;
    	property ShowHint;
    	property ParentShowHint;
    	property PopupMenu;

    	property OnClick;
    	property OnDblClick;
    	property OnDragDrop;
    	property OnDragOver;
    	property OnEndDrag;
    	property OnMouseDown;
    	property OnMouseMove;
    	property OnMouseUp;
    	property OnKeyUp;
    	property OnKeyDown;
  end;

procedure Register;

implementation


(*
 * About property
 *)
procedure TAboutProperty.Edit;
var
	msg: string;
const
	carriage_return = chr(13);
begin
	msg := 'IconButton Component v1.0';
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Copyright  1996 InforTech, Inc.  All Rights Reserved.');
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Developed by:');
  AppendStr(msg, carriage_return);  AppendStr(msg, carriage_return);
  AppendStr(msg, '  Steven Martins');
  AppendStr(msg, carriage_return);
  AppendStr(msg, '  E-Mail: steven.martins@workmail.com');
  AppendStr(msg, carriage_return);
  AppendStr(msg, '  Home Page:');
  AppendStr(msg, carriage_return);
  AppendStr(msg, '  http://www.geocities.com/SilIconValley/Park/7619/');
  AppendStr(msg, carriage_return);
  ShowMessage(msg);
end;

function TAboutProperty.GetAttributes: TPropertyAttributes;
begin
 	Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutProperty.GetValue: string;
begin
	Result := 'Click on...for About box';
end;


(*
 *  IconButton
 *)

Constructor TIconButton.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	AutoInitialize;
	Width := 85;
	Height := 75;
	TabStop := True;
end;

Destructor TIconButton.Destroy;
begin
	AutoDestroy;
	inherited Destroy;
end;

procedure TIconButton.AutoInitialize;
begin
	FIcon := TIcon.Create;
end;

procedure TIconButton.AutoDestroy;
begin
	FIcon.Free;
end;

procedure TIconButton.CMDialogChar(var Message: TCMDialogChar);
begin
	with Message do
	if IsAccel(CharCode, Caption) then
	begin
		if Enabled then Click;
  	Result := 1;
	end
	else inherited;
end;

procedure TIconButton.KeyUp(var Key: Word; Shift: TShiftState);
begin
	if Key = VK_SPACE then
	begin
		if Enabled then
		begin
			MouseUp(mbLeft, [], 1,1);
	  	Click;
		end;
	end;
	inherited KeyUp(Key, Shift);
end;

procedure TIconButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
	if Enabled = False then Exit;
	if Key = VK_SPACE then MouseDown(mbLeft,[],1,1);
	if Key = VK_RETURN then Click;
end;

procedure TIconButton.DoDrawText(cv: TCanvas; x,y,w,h: Integer);
var
	s: array [0..128] of Char;
	r: TRect;
begin
	StrPCopy(s,caption);
	r := Rect(x,y,w,h);
	if Enabled then
		DrawText(cv.handle,s,strlen(s), r, DT_CENTER or dt_vcenter or dt_singleline)
  else
  begin
		cv.font.color := clBtnShadow;
		Drawtext(cv.handle,s,strlen(s), r, DT_CENTER or dt_vcenter or dt_singleline);
  	r.left := r.left + 1;
  	r.top := r.top + 1;
  	r.right := r.right + 1;
  	r.bottom := r.bottom + 1;
  	cv.font.color := clbtnhighlight;
  	Drawtext(cv.handle,s,strlen(s), r, DT_CENTER or dt_vcenter or dt_singleline);
  end;
end;

procedure TIconButton.SetIcon(Value: TIcon);
begin
	FIcon.Assign(Value);
	Invalidate;
end;

procedure TIconButton.CMTextChanged(var msg: TMessage);
begin
	Paint;
end;

procedure TIconButton.CMEnabledChanged(var msg: TMessage);
begin
	Paint;
	inherited;
end;

procedure TIconButton.CMFocusChanged(var msg: TMessage);
begin
	Paint;
	inherited;
end;

procedure TIconButton.Click;
begin
	DrawUp;
	if Enabled then inherited Click;
end;

procedure TIconButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then
  begin
    SetFocus;
    if not FDown then
    begin
      FState := bsDown;
      DrawDown;
    end;
    FDragging := True;
  end;
end;

procedure TIconButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then
  begin
    if not FDown then NewState := bsUp;

    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
      if FDown then NewState := bsUp else NewState := bsDown;
    if NewState <> FState then
    begin
      FState := NewState;
      DrawDown;
    end;
    if NewState = bsUp then DrawUp;
  end;
end;

procedure TIconButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then
  begin
    FDragging := False;
    FState := bsUp;
    Paint;
  end;
end;

procedure TIconButton.Paint;
begin
	if Parent = nil then Exit;
	DrawUp;
end;

procedure TIconButton.SunkenBorder(cv: TCanvas; r: TRect);
begin
	with cv do
	begin
		brush.color := clBtnFace;
		fillrect(r);
		pen.Width := 1;
		pen.color := clBtnShadow;
		moveto(r.left,r.bottom);
		lineto(r.left,r.top);
		lineto(r.right,r.top);
		pen.color := clBtnHighlight;
		lineto(r.right,r.bottom);
		lineto(r.left,r.bottom);
  end;
end;

procedure TIconButton.SunkenBorderOutlineFill(cv: TCanvas;r: TRect; c:TColor);
begin
	with cv do
  begin
		brush.color := c;
		fillrect(r);
		pen.color := clbtnshadow;
		moveto(r.left,r.bottom);
		lineto(r.left,r.top);
		lineto(r.right,r.top);
		pen.color := clbtnhighlight;
		lineto(r.right,r.bottom);
		lineto(r.left - 1,r.bottom);
		pen.color := clblack;
		brush.style := bssolid;
		pen.Width := 1;
		pen.style := psinsideframe;
		rectangle(r.left + 1,r.top + 1,r.right,r.bottom);
		pen.color := clbtnface;
		moveto(r.left + 1,r.bottom - 1);
		lineto(r.right - 1,r.bottom - 1);
		lineto(r.right - 1,r.top);
  end;
end;

procedure TIconButton.RaisedBorderOutlineFill(cv: TCanvas; r:TRect; c:TColor);
begin
	with cv do
	begin
		brush.color := c;
		pen.color := clblack;
		brush.style := bssolid;
		pen.Width := 1;
		pen.style := psinsideframe;
		rectangle(r.left,r.top,r.right + 1,r.bottom + 1);
		pen.color := clbtnface;
		moveto(r.left,r.bottom - 1);
		lineto(r.left,r.top);
		lineto(r.right,r.top);
		pen.color := clbtnhighlight;
		moveto(r.left + 1,r.bottom - 1);
		lineto(r.left + 1,r.top + 1);
		lineto(r.right - 1,r.top + 1);
		pen.color := clbtnshadow;
		lineto(r.right - 1,r.bottom - 1);
		lineto(r.left,r.bottom - 1);
  end;
end;

procedure TIconButton.DrawUp;
var
	bitmap: TBitmap;
	x,y: Integer;
begin
	{bitmap used for double-buffering (no flicker)}
	bitmap := TBitmap.create;

	with bitmap do
	begin
		Width := self.Width;
		Height := self.Height;
    {outside edge}
		RaisedBorderOutlineFill(Canvas,rect(0,0,Width - 1,Height - 1),clbtnface);
    {inner Icon area}
		SunkenBorder(Canvas,rect(8,8,Width - 9,47));

		if focused then
		begin
			{Draw dark outline}
			Canvas.pen.color := clblack;
			Canvas.moveto(0,Height);
			Canvas.lineto(0,0);
      Canvas.lineto(Width,0);
  	end;

		if Icon.empty = False then
		begin
			{Draw Icon centered in inner area}
			x := (Width - Icon.Width) div 2;
			y := 12;
			Canvas.brush.color := clbtnface;
			DrawIcon(Canvas.handle,x,y,Icon.handle);
		end;

		{Draw caption}
		Canvas.font := font;
		x := (Width - Canvas.textWidth(caption)) div 2;
		y := (((Height - 48) - Canvas.textHeight(caption)) div 2) + 48;
		Canvas.brush.style := bsclear;
		DoDrawText(Canvas,0,y,Width,Canvas.textHeight(caption) + y);

		{Draw focus rect around caption if necessary}
		Canvas.brush.style := bssolid;
		if focused then
			Canvas.Drawfocusrect(rect(4,52,Width - 4,Height - 4));
  end;

	{Draw bitmap onto screen}
	Canvas.Draw(0,0,bitmap);
	{free bitmap}
  bitmap.free;
end;

procedure TIconButton.DrawDown;
var
	bitmap: TBitmap;
	x1,y1: Integer;
begin
	{bitmap used for double-buffering (no flicker)}
	bitmap := TBitmap.Create;

	with bitmap do
	begin
		Width := self.Width;
		Height := self.Height;
		{outside edge}
		SunkenBorderOutlineFill(Canvas,rect(0,0,Width - 1,Height - 1),clbtnface);
		{inner Icon area}
		SunkenBorder(Canvas,rect(9,9,Width - 8,48));

		if Icon.empty = False then
		begin
			{center Icon in inner area}
      x1 := ((Width - Icon.Width)div 2) + 1;
			y1 := 13;
			Canvas.brush.color := clbtnface;
			DrawIcon(Canvas.handle,x1,y1,Icon.handle);
  	end;

		{Draw caption}
		Canvas.font := font;
		x1 := ((Width - Canvas.TextWidth(caption))div 2) + 1;
		y1 := (((Height - 48) - Canvas.TextHeight(caption))div 2) + 49;
		Canvas.brush.style := bsclear;
		DoDrawText(Canvas,1,y1,Width + 1,Canvas.TextHeight(caption) + y1);

		{Draw focus rect around caption}
		Canvas.brush.style := bssolid;
		if focused then
		Canvas.Drawfocusrect(rect(5,53,Width - 3,Height - 3));
  end;

	{Draw bitmap on screen}
	Canvas.Draw(0,0,bitmap);
	{free bitmap}
	bitmap.free;
end;

procedure Register;
begin
	RegisterComponents('InforTech', [TIconButton]);
  {Register the about box property editor}
  RegisterPropertyEditor(TypeInfo(TAboutProperty),
      TIconButton, 'ABOUT', TAboutProperty);
end;

end.
